part of tecfy_core_client_package;

class EntityHistoryPage extends StatefulWidget {
  final String entity;
  final String id;

  EntityHistoryPage(
    this.entity,
    this.id, {
    Key? key,
  }) : super(key: key);
  @override
  _EntityHistoryPageState createState() => _EntityHistoryPageState();
}

class _EntityHistoryPageState extends AppState<EntityHistoryPage> {
  List<Map<String, dynamic>>? _data;
  bool _isMobile = false;

  @override
  void initState() {
    super.initState();

    TecfyBasicApp.apiService
        .httpGetDynamic(
            'EntityLog/GetLogs?entity=${widget.entity}&id=${widget.id}&actions=Add,Update,Delete')
        .then((result) {
      setState(() => _data = result.data
          .map<Map<String, dynamic>>((e) => e as Map<String, dynamic>)
          .toList());
    });
  }

  @override
  Widget build(BuildContext context) {
    _isMobile = LayoutService.isMobile(context);
    return BasePage(
      context,
      trans('History'),
      widget.entity,
      isMobile: _isMobile,
      child: Align(
        alignment: Alignment.topCenter,
        child: Container(
          constraints: BoxConstraints(maxWidth: ThemeService.containerWidth),
          child: _data == null
              ? Center(child: CircularProgressIndicator())
              : Scrollbar(
                  isAlwaysShown: true,
                  showTrackOnHover: true,
                  child: SingleChildScrollView(
                    padding: EdgeInsets.all(20),
                    scrollDirection: Axis.horizontal,
                    child: _getTable(),
                  )),
        ),
      ),
    );
  }

  Widget _getTable() {
    return AppDataTable(
      onCellClicked: (data, filed) {
        if (filed == 'action') {
          AppNavigator.showAppDialogEx(
              context,
              () => EntityFormPage(
                    widget.entity,
                    data: jsonDecode(data['model']),
                    isPopup: true,
                    readOnly: true,
                    loadData: false,
                  ),
              closeButton: true,
              size: DialogBoxSizes.large);
        }
        if (filed == 'success') {
          AppNavigator.showMessage(context, 'Result is:',
              message: data['result'].toString());
        }
        if (filed == 'deviceType') {
          AppNavigator.showMessage(context, 'IP Address is:',
              message: data['ipAddress'].toString());
        }
        if (filed == 'timestamp') {
          AppNavigator.showMessage(
              context, 'Response within ${data['millisecond']} millisecond');
        }
      },
      //isSelectedField: 'selected',
      readOnly: true,
      columns: [
        PropertyText(
            dbName: 'action',
            nameAr: 'الحدث',
            nameEn: 'Action',
            readOnly: true),
        PropertyText(
            dbName: 'userFullName',
            nameAr: 'المستخم',
            nameEn: 'User',
            readOnly: true),
        PropertyDate(
            dbName: 'timestamp',
            nameAr: 'التاريخ',
            nameEn: 'Date',
            format: 'yyyy-MM-dd    HH:mm',
            readOnly: true),
        PropertyText(
            dbName: 'clientVersion',
            nameAr: 'الاصدار',
            nameEn: 'Version',
            readOnly: true),
        PropertyText(
            dbName: 'deviceType',
            nameAr: 'الجهاز',
            nameEn: 'Device',
            readOnly: true),
        PropertyBoolean(
            dbName: 'success',
            nameAr: 'بنجاح',
            nameEn: 'Success',
            readOnly: true),
      ],
      data: _data!,
    );
  }
}
