part of tecfy_core_client_package;

class ReportDetailPage extends StatefulWidget {
  /// the report name
  final dynamic report;

  /// the filter properties
  final List<EntityFilter> filter;

  /// view report page with filter and print
  ReportDetailPage(this.report, this.filter);
  @override
  _ReportDetailPageState createState() => _ReportDetailPageState();
}

class _ReportDetailPageState extends AppState<ReportDetailPage> {
  List<Map<String, dynamic>>? _data;
  List<Property>? _filters;
  Map<String, dynamic>? _filterData;
  Map<String, dynamic>? _totals;
  int? _sortColumnIndex;
  bool _sortAcd = true;
  ByteData? _font;
  late List<ReportProperty> _props;
  late List<ReportProperty> _propsSortedForPdf;
  late ScrollController _scrollControllerH;
  late ScrollController _scrollControllerV;
  String _mode = "portrait";
  @override
  void initState() {
    _scrollControllerV = ScrollController();
    _scrollControllerH = ScrollController();
    rootBundle.load("assets/fonts/STC-Regular.ttf").then((d) => _font = d);
    _props = widget.report['properties']
        .map<ReportProperty>((p) => ReportProperty.fromMap(p))
        .toList();
    if (LocalConfigService.isArabic)
      _propsSortedForPdf = _props.reversed.toList();
    else
      _props = _props.toList();
    _getData();
    super.initState();
  }

  @override
  void dispose() {
    _scrollControllerH.dispose();
    _scrollControllerV.dispose();
    super.dispose();
  }

  void _onSort(String dbName, ValueTypes type) {
    _data?.sort((a, b) => _sortAcd
        ? (type == ValueTypes.Number
            ? (a[dbName] - b[dbName]).toInt()
            : a[dbName].toString().compareTo(b[dbName].toString()))
        : (type == ValueTypes.Number
            ? (b[dbName] - a[dbName]).toInt()
            : b[dbName].toString().compareTo(a[dbName].toString())));
    setState(() {});
  }

  Future _getData() async {
    try {
      //List<dynamic> data;
      // if (ConnectionService.isOfflineMode) {
      //   var result = await EntityService().getData(
      //       EntityService.getEntity(widget.report['entityName']),
      //       filter: widget.filter,
      //       allRecords: true);
      //   data = result.data;
      // } else {
      if (TecfyCoreApp.reportService != null &&
          TecfySyncService.isOfflineMode &&
          widget.report['isOnlineOnly'] == false) {
        _data = (await TecfyCoreApp.reportService
                ?.getReportData(widget.report['dbName'], widget.filter))
            as List<Map<String, dynamic>>?;
      } else {
        var result = await TecfyBasicApp.apiService.httpGetDynamic(
            'Report/GetData/${widget.report['dbName']}?extraData=true' +
                (widget.filter.length > 0
                    ? '&filter=${jsonEncode(widget.filter.map<Map>((f) => f.toJson()).toList(), toEncodable: EntityService.customEncode)}'
                    : ''));
        _data = result.data['data']
            .map<Map<String, dynamic>>((e) => e as Map<String, dynamic>)
            .toList();
        _filters = result.data['filter']
            .map<Property>((e) => Property.getProperty(e))
            .toList();
        _filterData = result.data['filterData'] as Map<String, dynamic>;
      }
      // }
      if (_data != null) {
        // if (widget.report['totals'] != null &&
        //     widget.report['totals'].length > 0) {
        //   totals = {};
        //   for (var t in widget.report['totals']) {
        //     totals[t] = 0;
        //     for (var d in data) {
        //       totals[t] += d[t] ?? 0;
        //     }
        //   }
        // }
        _totals = _data?.removeLast();

        setState(() {});
      } else {
        await AppNavigator.showMessage(context, "Error or No Data!");
        Navigator.pop(context);
      }
    } catch (err, trace) {
      print('XXXXXXXXXXXXXXXXXXX report load error');
      print(err);
      print(trace);
      await AppNavigator.showMessage(context, "Error getting data!");
      Navigator.pop(context);
    }
  }

  List<DataRow> _getRows() {
    return _data!
        .map<DataRow>((d) => DataRow(
              cells: _props.map<DataCell>((p) => _getCell(p, d)).toList(),
            ))
        .toList()
      ..add(DataRow(
        cells: _props
            .map<DataCell>((p) => _getCell(p, _totals, totals: true))
            .toList(),
      ));
  }

  @override
  Widget build(BuildContext context) {
    return BasePage(context,
        widget.report['name${LocalConfigService.languagePrefex}'], 'report',
        isMobile: LayoutService.isMobile(context),
        appBarActions: [
          AppButtonIconText(
            label: 'Print',
            icon: Icon(Icons.print, color: Colors.white),
            onPressed: _onPrint,
          ),
          SizedBox(width: 5),
          AppButtonIconText(
            label: 'PDF',
            icon: Icon(Icons.picture_as_pdf, color: Colors.white),
            onPressed: _onSharePDF,
          ),
          PopupMenuButton<String>(
            onSelected: (val) {
              setState(() {
                _mode = val;
              });
            },
            itemBuilder: (BuildContext context) => [
              PopupMenuItem<String>(
                value: 'portrait',
                child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      Checkbox(
                        value: _mode == 'portrait' ? true : false,
                        onChanged: (value) {},
                        activeColor: Colors.black54,
                      ),
                      Text("portrait",
                          style: TextStyle(
                              fontSize: 15, fontWeight: FontWeight.bold)),
                    ]),
              ),
              PopupMenuItem<String>(
                value: 'landscap',
                child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      Checkbox(
                        value: _mode == 'landscap' ? true : false,
                        onChanged: (value) {},
                        activeColor: Colors.black54,
                      ),
                      Text("Landscap",
                          style: TextStyle(
                              fontSize: 15, fontWeight: FontWeight.bold)),
                    ]),
              ),
            ],
          ),
        ],
        child: Column(
          children: [
            _getFilterTable(),
            Expanded(child: _pageContent()),
          ],
        ));
  }

  Future<dynamic> _onPrint() async {
    var resultData = await EntityService.getData(
      EntityService.getEntity("Branch"),
    );
    if (resultData.data == null) return;
    var branchLogo =
        await ImageService.getImageBytes(resultData.data![0]["logoId"]);
    var doc = _getPrintablePdf(
        resultData.data![0]["printTitleAr"] ?? '',
        resultData.data![0]["printSubTitleAr"] ?? '',
        resultData.data![0]["printFooterAr"] ?? '',
        resultData.data![0]["printSubFooterAr"] ?? '',
        branchLogo);
    var result = await Printing.layoutPdf(
      onLayout: (format) async => doc.save(),
      //name:widget.report['name${LocalConfigService.languagePrefex}'],
    );
    print(result);
  }

  Future<dynamic> _onSharePDF() async {
    var resultData = await EntityService.getData(
      EntityService.getEntity("Branch"),
    );
    if (resultData.data == null) return;
    var branchLogo =
        await ImageService.getImageBytes(resultData.data![0]["logoId"]);
    var doc = _getPrintablePdf(
      resultData.data![0]["printTitleAr"] ?? '',
      resultData.data![0]["printSubTitleAr"] ?? '',
      resultData.data![0]["printFooterAr"] ?? '',
      resultData.data![0]["printSubFooterAr"] ?? '',
      branchLogo,
    );
    var result = await Printing.sharePdf(
        bytes: await doc.save(), filename: 'nameEn.pdf');
    // var result = await Printing.layoutPdf(
    //   onLayout: (format) async => doc.save(),
    //   //name:widget.report['name${LocalConfigService.languagePrefex}'],
    // );
    print(result);
  }

  pw.Document _getPrintablePdf(
      var titleAr, var subTitleAr, var footerAr, var subFooterAr, var logoId) {
    final doc = pw.Document();
    DateTime printingDate = DateTime.now();
    String formattedDate =
        intl.DateFormat('yyyy-MM-dd   kk:mm').format(printingDate);
    doc.addPage(
      pw.MultiPage(
        orientation: _mode == "landscap"
            ? pw.PageOrientation.landscape
            : pw.PageOrientation.portrait,
        pageFormat: PdfPageFormat.a4,
        textDirection: pw.TextDirection.rtl,
        margin: pw.EdgeInsets.all(5),
        footer: (context) =>
            pw.Row(mainAxisAlignment: pw.MainAxisAlignment.center, children: [
          pw.Row(children: [
            pw.Text(" ${formattedDate.toString()} ",
                style: pw.TextStyle(font: pw.Font.ttf(_font!)),
                textDirection: pw.TextDirection.ltr),
            pw.Text('تاريخ الطباعة : ',
                style: pw.TextStyle(font: pw.Font.ttf(_font!)),
                textDirection: pw.TextDirection.rtl),
          ]),
          pw.SizedBox(width: 40),
          pw.Row(children: [
            pw.Text(" ${LocalConfigService.user!.fullName.toString()} ",
                style: pw.TextStyle(font: pw.Font.ttf(_font!)),
                textDirection: pw.TextDirection.rtl),
            pw.Text('أسم المستخدم : ',
                style: pw.TextStyle(font: pw.Font.ttf(_font!)),
                textDirection: pw.TextDirection.rtl),
          ]),
        ]),
        build: (c) => [
          pw.Padding(
              padding: pw.EdgeInsets.only(top: 0),
              child: pw.Column(children: [
                pw.Row(children: [
                  pw.Expanded(
                      child: pw.Row(children: [
                    logoId == null
                        ? pw.Container(width: 120, height: 120)
                        : pw.Container(
                            width: 120,
                            height: 120,
                            decoration: pw.BoxDecoration(
                                image: pw.DecorationImage(
                                    image: pw.MemoryImage(logoId),
                                    fit: pw.BoxFit.contain)),
                          ),
                    pw.Container(width: 40),
                  ])),
                  pw.SizedBox(width: 20),
                  pw.Expanded(
                      child: pw.Column(
                    crossAxisAlignment: pw.CrossAxisAlignment.end,
                    children: [
                      pw.SizedBox(height: 5),
                      pw.Text(
                        titleAr.toString(),
                        style: pw.TextStyle(
                            font: pw.Font.ttf(_font!),
                            fontWeight: pw.FontWeight.bold,
                            fontSize: 16),
                        // textDirection: pw.TextDirection.rtl
                      ),
                      pw.Text(subTitleAr.toString(),
                          style: pw.TextStyle(font: pw.Font.ttf(_font!)),
                          textDirection: pw.TextDirection.rtl),
                      pw.Text(footerAr.toString(),
                          style: pw.TextStyle(font: pw.Font.ttf(_font!)),
                          textDirection: pw.TextDirection.rtl),
                      pw.Text(subFooterAr.toString(),
                          style: pw.TextStyle(font: pw.Font.ttf(_font!)),
                          textDirection: pw.TextDirection.rtl),
                    ],
                  )),
                ]),
              ])),
          pw.Center(
              child: pw.Text(
                  widget.report['name${LocalConfigService.languagePrefex}'],
                  style:
                      pw.TextStyle(font: pw.Font.ttf(_font!), fontSize: 15))),
          pw.SizedBox(height: 20),
          _getFilterTablePdf(),
          pw.Table(
              //columnWidths: sortColumnIndex,
              border: pw.TableBorder(
                  horizontalInside: pw.BorderSide(),
                  top: pw.BorderSide(),
                  bottom: pw.BorderSide(),
                  right: pw.BorderSide(),
                  left: pw.BorderSide()),
              children: (_data
                      ?.map<pw.TableRow>((d) => pw.TableRow(
                            children: _propsSortedForPdf
                                .map<pw.Widget>((p) => _getCellReport(p, d))
                                .toList(),
                          ))
                      .toList() ??
                  [])
                ..insert(
                  0,
                  pw.TableRow(
                    decoration: pw.BoxDecoration(color: PdfColors.grey300),
                    children: _propsSortedForPdf
                        .map<pw.Widget>(
                          (p) => pw.Padding(
                            padding: pw.EdgeInsets.only(left: 5, right: 5),
                            child: pw.Align(
                              alignment: LocalConfigService.isArabic
                                  ? pw.Alignment.centerRight
                                  : pw.Alignment.centerLeft,
                              child: pw.Text(
                                p.name,
                                style: pw.TextStyle(font: pw.Font.ttf(_font!)),
                              ),
                            ),
                          ),
                        )
                        .toList(),
                  ),
                )
                ..add(pw.TableRow(
                    decoration: pw.BoxDecoration(color: PdfColors.grey200),
                    children: _propsSortedForPdf
                        .map<pw.Widget>(
                            (p) => _getCellReport(p, _totals, totals: true))
                        .toList()))),
        ],
      ),
    );

    return doc;
  }

  pw.Widget _getCellReport(ReportProperty prop, data, {totals: false}) {
    var val = data[prop.dbName];
    PdfColor color = PdfColors.black;
    if (prop.colorField?.isNotEmpty ?? false) {
      try {
        color = hexToColorPdf(data[prop.colorField]);
      } catch (err) {
        print(err);
      }
    }
    if (totals) {
      if (val == null || val == 0 || val == '' || val == '0')
        return pw.Container();
    } else {
      if (val == null) return pw.Container();
    }
    var textStyle = totals
        ? pw.TextStyle(
            font: pw.Font.ttf(_font!),
            fontSize: 13,
            fontWeight: pw.FontWeight.bold)
        : pw.TextStyle(font: pw.Font.ttf(_font!), color: color);
    pw.Widget cell;
    switch (prop.type) {
      case ValueTypes.Text:
        cell = pw.Text(val, style: textStyle);
        break;
      case ValueTypes.Number:
        if (prop.format == '0')
          return pw.Text((val as num).toInt().toString(), style: textStyle);
        cell = pw.Text((val as num).toDouble().toStringAsFixed(2),
            style: textStyle);
        break;
      case ValueTypes.DateTime:
        DateTime? date;
        if (val is num)
          date = DateTime.fromMillisecondsSinceEpoch(val.toInt()).toLocal();
        if (val is String) date = BaseModel.parseUTC(val)!;
        if (val is DateTime) date = val;

        cell = pw.Text(
            intl.DateFormat(prop.format ?? "yyyy-MM-dd HH:mm")
                .format(date!.toLocal()),
            style: textStyle);

        // cell = pw.Text(val.toString().replaceFirst('T', ' ').substring(0, 16),
        //     style: textStyle);
        break;
      default:
        cell = pw.Text('-');
        break;
    }
    return pw.Padding(
        padding: pw.EdgeInsets.only(left: 5, right: 5),
        child: pw.Align(
            alignment: LocalConfigService.isArabic
                ? pw.Alignment.centerRight
                : pw.Alignment.centerLeft,
            child: cell));
  }

  pw.Widget _getFilterTablePdf() {
    if (_filters == null) return pw.Container();
    return pw.Container(
        constraints: pw.BoxConstraints(maxWidth: 350),
        child: pw.Column(
          children: _filters!
              .map((e) => pw.Row(
                    children: [
                      pw.Expanded(
                          flex: 4,
                          child: pw.Text(e.getDisplayText(_filterData ?? {}),
                              style: pw.TextStyle(font: pw.Font.ttf(_font!)))),
                      pw.Expanded(
                          flex: 3,
                          child: pw.Text(
                            e.nameAr,
                            style: pw.TextStyle(font: pw.Font.ttf(_font!)),
                          )),
                    ],
                  ))
              .toList(),
        ));
  }

  Widget _pageContent() {
    return _data == null
        ? Center(
            child: CircularProgressIndicator(
                valueColor:
                    AlwaysStoppedAnimation<Color>(ThemeService.mainColor)))
        // : data != null
        //     ? PdfPreview(build: (c) => getPrintablePdf().save())
        : Scrollbar(
            isAlwaysShown: true,
            showTrackOnHover: true,
            controller: _scrollControllerH,
            child: SingleChildScrollView(
              controller: _scrollControllerH,
              scrollDirection: Axis.horizontal,
              child: Scrollbar(
                isAlwaysShown: true,
                showTrackOnHover: true,
                controller: _scrollControllerV,
                child: SingleChildScrollView(
                  controller: _scrollControllerV,
                  scrollDirection: Axis.vertical,
                  child: DataTable(
                      showCheckboxColumn: true,
                      sortColumnIndex: _sortColumnIndex,
                      sortAscending: _sortAcd,
                      columns: _props
                          .map<DataColumn>((p) => DataColumn(
                              label: Text(p.name),
                              numeric: p.type == ValueTypes.Number,
                              onSort: (s, d) {
                                _sortColumnIndex = s;
                                _sortAcd = d;
                                _onSort(p.dbName, p.type);
                              },
                              tooltip: p.nameEn))
                          .toList(),
                      rows: _getRows()
                      // ..insert(0, TableRow(
                      //   children: widget.report['properties'].map<TableCell>((p)=>TableCell(child: Text(p['name${LocalConfigService.languagePrefex}'] ?? 'N/A', style: ThemeService.titleStyle))).toList(),
                      // )
                      //)
                      ),
                ),
              ),
            ),
          );
  }

  Color hexToColor(String code) {
    return new Color(int.parse(code.substring(1, 7), radix: 16) + 0xFF000000);
  }

  PdfColor hexToColorPdf(String code) {
    return new PdfColor.fromHex(code);
  }

  DataCell _getCell(ReportProperty prop, data, {totals: false}) {
    var val = data[prop.dbName];
    dynamic onTap;

    if (prop.drillDownEntity != null && prop.drillDownIdField != null) {
      onTap = () async {
        AppNavigator.showLoading(context);
        var entity = prop.drillDownEntity!;
        if (prop.drillDownEntityDynamic) entity = data[prop.drillDownEntity!];
        var d =
            await EntityService.getOne(entity, data[prop.drillDownIdField!]);
        if (d == null) {
          AppNavigator.pop(context);
          return;
        }
        await AppNavigator.showAppDialogEx(
            context,
            () => EntityFormPage(
                  entity,
                  data: d,
                  isPopup: true,
                ),
            closeButton: false,
            size: DialogBoxSizes.large);
        AppNavigator.pop(context);
        return;
      };
    }
    Color color = Colors.black;
    if (prop.colorField?.isNotEmpty ?? false) {
      try {
        color = hexToColor(data[prop.colorField]);
      } catch (err) {}
    }
    if (totals) {
      if (val == null || val == 0 || val == '' || val == '0')
        return DataCell(Container());
    } else {
      if (val == null) return DataCell(Text('.'));
    }
    switch (prop.type) {
      case ValueTypes.Text:
        return DataCell(
            Text(val,
                style: totals
                    ? ThemeService.titleStyle
                    : TextStyle(fontSize: 12, color: color)),
            onTap: onTap);
      case ValueTypes.Number:
        if (val is String) {
          val = num.parse(val);
        }
        if (prop.format == '0')
          return DataCell(Text((val as num).toInt().toString(),
              style: totals
                  ? ThemeService.titleStyle
                  : TextStyle(fontSize: 12, color: color)));
        else
          return DataCell(Text((val as num).toDouble().toStringAsFixed(2),
              style: totals
                  ? ThemeService.titleStyle
                  : TextStyle(fontSize: 12, color: color)));
      case ValueTypes.DateTime:
        DateTime? date = BaseModel.getDateEx(val);
        // if (val is num)
        //   date = DateTime.fromMillisecondsSinceEpoch(val.toInt()).toLocal();
        // if (val is String) date = BaseModel.parseUTC(val)!;
        // if (val is DateTime) date = val;

        return DataCell(Text(
            date == null
                ? ''
                : intl.DateFormat(prop.format ?? "yyyy-MM-dd HH:mm")
                    .format(date.toLocal()),
            style: totals
                ? ThemeService.titleStyle
                : TextStyle(fontSize: 10, color: color)));
      default:
        return DataCell(Text('-'));
    }
  }

  Widget _getFilterTable() {
    if (_filters == null) return Container();
    return Container(
        constraints: BoxConstraints(maxWidth: 350),
        child: Column(
          children: _filters!
              .map((e) => Row(
                    children: [
                      Expanded(flex: 3, child: Text(e.nameAr)),
                      Expanded(
                          flex: 4,
                          child: Text(e.getDisplayText(_filterData ?? {}))),
                    ],
                  ))
              .toList(),
        ));
  }

  _lowerFirst(String s) {
    return '${s[0].toLowerCase()}${s.substring(1)}';
  }
}
